/*
  Simple socket client example

  Jim Teresco
  Department of Computer Science
  Williams College

  Last Modification:
  Thu Feb 27 09:24:46 EST 2003

  $Id: client.c,v 1.1 2003/02/27 14:47:55 terescoj Exp $
*/

#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include <unistd.h>
#include <string.h>
#include <netinet/in.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <arpa/inet.h>

#include "name_to_addr.h"

char *my_id="Jim Teresco's version of the socket client";

int main(int argc, char *argv[]) {
  int send_port=-1;
  char remote_host[100];
  char buf[100];
  int i;
  short size, convsize, array[100];

  /* for getopt */
  extern char *optarg;
  extern int optind, opterr;
  char c;

  /* socket stuff */
  int clientfd, result;
  struct sockaddr_in serveraddress;
  struct sockaddr *serverptr;
  struct in_addr ipaddress;

  strcpy(remote_host,"");
  /* parse command line */
  while ((c=getopt(argc, argv, "p:h:")) != -1) {
    switch (c) {
    case 'p':
      /* -p: set send_port */
      send_port=atoi(optarg);
      break;
    case 'h':
      /* -h: set remote hostname */
      strcpy(remote_host,optarg);
      break;
    }
  }

  if (send_port < 0) {
    fprintf(stderr,"%s: -p must be used to specify port number\n",argv[0]);
    exit(1);
  }

  if (strcmp(remote_host,"")==0) {
    fprintf(stderr,"%s: -h must be used to specify remote host\n",argv[0]);
    exit(1);
  }

  ipaddress.s_addr=name_to_addr(remote_host);
  if (!ipaddress.s_addr) {
    fprintf(stderr,"%s: host not found: %s\n",argv[0],remote_host);
    exit(1);
  }

  printf("Attempting connection to %s[%s] port %d\n",
	 remote_host, inet_ntoa(ipaddress), send_port);

  /* set up the socket to attempt connection */

  serverptr=(struct sockaddr *)&serveraddress;
  bzero(&serveraddress,sizeof(struct sockaddr_in));
  serveraddress.sin_family=AF_INET;
  serveraddress.sin_addr.s_addr=ipaddress.s_addr;
  serveraddress.sin_port=htons(send_port);

  clientfd=socket(AF_INET, SOCK_STREAM, 0);

  do {
    printf("calling connect...\n");
    result=connect(clientfd,serverptr,sizeof(struct sockaddr_in));
    if (result == -1) sleep(1);
  } while (result==-1);

  /* we are connected ! */
  /* first send identification string */

  write(clientfd,my_id,strlen(my_id)+1);  /* be sure to send the NULL too! */

  c=' ';
  while (c!='q') {
    printf("\nNext? ");
    gets(buf);
    c=buf[0];
    printf(" sending command '%c'\n",c);
    write(clientfd,&c,1);
    switch (c) {
      /* send any additional information here */
#if 0  /* this is an odd way to comment things out */
    case 'a':
      /* an integer followed by a an array of shorts */
      size=10; convsize=htons(size);
      for (i=0;i<size;i++) array[i]=htons(i);
      write(clientfd,&convsize,sizeof(short));
      write(clientfd,array,size*sizeof(short));
      break;
#endif
    case 'i':
      /* resend our identity */
      write(clientfd,my_id,strlen(my_id)+1);
      break;
    }
  }

  close(clientfd);
  return 0;
}
