/*
  Simple socket server example

  Jim Teresco
  Department of Computer Science
  Williams College

  Last Modification:
  Thu Feb 27 09:24:46 EST 2003

  $Id: server.c,v 1.1 2003/02/27 14:47:55 terescoj Exp $
*/

#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include <unistd.h>
#include <string.h>
#include <ctype.h>
#include <netinet/in.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <sys/time.h>

#include "read_line.h"

void process_client(int, char *, int);

int main(int argc, char *argv[]) {
  int listen_port=-1;

  /* for getopt */
  extern char *optarg;
  extern int optind, opterr;
  char c;

  int sessions = 1; 

  /* socket stuff */
  int clientfd, serverfd, clientlen;
  struct sockaddr_in serveraddress;
  struct sockaddr_in clientaddress;
  struct sockaddr *serverptr;
  struct sockaddr *clientptr;

  /* parse command line */
  while ((c=getopt(argc, argv, "p:")) != -1) {
    switch (c) {
    case 'p':
      /* -p: set listen_port */
      listen_port=atoi(optarg);
      break;
    }
  }

  if (listen_port < 0) {
    fprintf(stderr,"%s: -p must be used to specify port number\n",argv[0]);
    exit(1);
  }

  /* Ignore death of child */
  signal(SIGCHLD, SIG_IGN);

  /* set up AF_INET listen socket */
  serverptr=(struct sockaddr *)&serveraddress;
  serverfd=socket(AF_INET, SOCK_STREAM, 0);
  if (serverfd == -1) {
    perror("socket");
    exit(1);
  }
  bzero(&serveraddress,sizeof(struct sockaddr_in));
  serveraddress.sin_family=AF_INET;
  serveraddress.sin_addr.s_addr=INADDR_ANY; /*htonl(INADDR_ANY);*/
  serveraddress.sin_port=htons(listen_port);

  if (bind(serverfd, serverptr, sizeof(struct sockaddr_in))==-1) {
    perror("bind");
    exit(1);
  }
  if (listen(serverfd, 5)==-1) {
    perror("listen");
    exit(1);
  }

  /* set up AF_INET client socket */
  clientptr=(struct sockaddr *)&clientaddress;
  
  printf("Server ready to accept...port %d\n",listen_port);
  while (1) {
    clientlen=sizeof(struct sockaddr_in);
    clientfd=accept(serverfd, clientptr, &clientlen);

    if (clientfd==-1) {
      perror("accept");
      exit(1);
    }

    if (fork() == 0) { /* new child to talk to this client */
      /* announce successful connection */
      printf("Accepted connection from %s\n",inet_ntoa(clientaddress.sin_addr));
      /* talk to this client */
      process_client(clientfd, inet_ntoa(clientaddress.sin_addr), sessions);
      close(clientfd);
      exit(0);
    } 
    else {
      sessions++;
      close(clientfd); /* close the client in the parent */
    }
  }
  return 0;
}

/* process info from a specific client */
void process_client(int fd, char *from, int sessions) {
  char messagebuf[100];
  char c;
  int size, i;
  short *array;

  /* first, an identifier string is sent, so read that */
  read_line(fd, messagebuf);
  printf("%s: Identified as <%s>\n",from,messagebuf);

  while (1) {
    read(fd,&c,1);
    switch (c) {
#if 0 /* this is an odd way to comment things out */
    case 'a':
      /* array of integers, first being a count */
      /* client is sending an array of shorts */
      read(fd, &size, sizeof(short));
      size = ntohs(size);
      array = (short *)malloc(sizeof(short) * size);
      read(fd, array, sizeof(short) * size);
      printf("%s: An array of %d integers received.\n[", from, size);
      for (i=0; i<size; i++)
      {
        array[i] = ntohs(array[i]);
        /*printf("%d ", array[i]);*/
      }
      printf("]\n");
      free(array); 
      break;
#endif
    case 'i':
      read_line(fd, messagebuf);
      printf("%s: Re-identified as <%s>\n",from,messagebuf);
      break;
    case 'q':
      /* client is terminating */
      printf("%s: Terminated.\n",from);
      return;
    default:
      /* unknown command */
      printf("%s: unknown command '%c'\n",from,c);
      break;
    }
  }
}
